# -*- coding: utf-8 -*-
"""
    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
    :contact: l.mierzwa@gmail.com
"""


class ExitCodes:

    ok = 0

    command_error = 1

    notfound_error = 2

    missing_config = 3

    user_error = 4

    auth_error = 5



# -*- coding: utf-8 -*-
"""
    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
    :contact: l.mierzwa@gmail.com
"""


from __future__ import unicode_literals

from plumbum import cli

from slumber.exceptions import SlumberHttpBaseException

from upaas.cli.base import UPaaSApplication

from upaas_client.main import ClientApplication
from upaas_client.return_codes import ExitCodes


@ClientApplication.subcommand('edit')
class Edit(UPaaSApplication):

    DESCRIPTION = "Edit running application settings"

    workers_min = 0
    workers_max = 0

    @cli.switch(["w", "--workers-min"], int, help="Minimum number of workers")
    def set_workers_min(self, value):
        self.workers_min = value

    @cli.switch(["W", "--workers-max"], int, help="Maximum number of workers")
    def set_workers_max(self, value):
        self.workers_max = value

    def main(self, name):
        self.setup_logger()

        if not self.workers_min and not self.workers_max:
            self.log.error("To edit application provide new minimum or "
                           "maximum workers count")
            return ExitCodes.user_error
        if self.workers_min and self.workers_max \
                and self.workers_min > self.workers_max:
            self.log.error("Maximum number of workers must be higher than "
                           "minimum")
            return ExitCodes.user_error

        self.log.info("Getting app '%s' details" % name)

        self.api_connect(self.parent.config.server.login,
                         self.parent.config.server.apikey,
                         self.parent.config.server.url)

        try:
            resp = self.api.application.get(name=name)
        except SlumberHttpBaseException as e:
            self.handle_error(e)
            if e.response.status_code == 401:
                return ExitCodes.auth_error
        else:
            if not resp.get('objects'):
                self.log.error("No such application registered: %s" % name)
                return ExitCodes.notfound_error
        app = resp['objects'][0]

        try:
            resp = self.api.run_plan.get(application=app['id'])
        except SlumberHttpBaseException as e:
            self.handle_error(e)
        else:
            if not resp.get('objects'):
                self.log.error("Application is stopped")
                return ExitCodes.user_error
        run_plan = resp['objects'][0]

        payload = {}
        if self.workers_min:
            payload['workers_min'] = self.workers_min
        if self.workers_max:
            payload['workers_max'] = self.workers_max
        try:
            self.api.run_plan(run_plan['id']).patch(payload)
        except SlumberHttpBaseException as e:
            self.handle_error(e)
            return ExitCodes.command_error

        try:
            self.api.application(app['id']).update.put(
                {'name': app['name']})
        except SlumberHttpBaseException as e:
            self.handle_error(e)
        else:
            self.log.info("Update task queued")



# -*- coding: utf-8 -*-
"""
    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
    :contact: l.mierzwa@gmail.com
"""


from __future__ import unicode_literals

from plumbum import cli

from slumber.exceptions import SlumberHttpBaseException

from upaas.cli.base import UPaaSApplication

from upaas_client.main import ClientApplication
from upaas_client.return_codes import ExitCodes


@ClientApplication.subcommand('start')
class Start(UPaaSApplication):

    DESCRIPTION = "Start application"

    workers_min = 1
    workers_max = 1

    @cli.switch(["w", "--workers-min"], int, help="Minimum number of workers")
    def set_workers_min(self, value):
        self.workers_min = value

    @cli.switch(["W", "--workers-max"], int, help="Maximum number of workers")
    def set_workers_max(self, value):
        self.workers_max = value

    def main(self, name):
        self.setup_logger()

        if self.workers_min and self.workers_max \
                and self.workers_min > self.workers_max:
            self.log.error("Maximum number of workers must be higher than "
                           "minimum")
            return ExitCodes.user_error

        self.log.info("Getting app '%s' details" % name)

        self.api_connect(self.parent.config.server.login,
                         self.parent.config.server.apikey,
                         self.parent.config.server.url)

        try:
            resp = self.api.application.get(name=name)
        except SlumberHttpBaseException as e:
            self.handle_error(e)
        else:
            if not resp.get('objects'):
                self.log.error("No such application registered: %s" % name)
                return ExitCodes.notfound_error

            app = resp['objects'][0]

            try:
                resp = self.api.run_plan.get(application=app['id'])
            except SlumberHttpBaseException as e:
                self.handle_error(e)
            else:
                if resp.get('objects'):
                    self.log.error("Application is already started")
                    return ExitCodes.user_error

            try:
                self.api.run_plan.post({
                    'application': app['id'],
                    'workers_min': self.workers_min,
                    'workers_max': self.workers_max
                })
            except SlumberHttpBaseException as e:
                self.handle_error(e)
                return ExitCodes.command_error

            try:
                self.api.application(app['id']).start.put(
                    {'name': app['name']})
            except SlumberHttpBaseException as e:
                self.handle_error(e)
            else:
                self.log.info("Start task queued")



# -*- coding: utf-8 -*-
"""
    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
    :contact: l.mierzwa@gmail.com
"""


from __future__ import unicode_literals

from slumber.exceptions import SlumberHttpBaseException

from upaas.cli.base import UPaaSApplication

from upaas_client.main import ClientApplication
from upaas_client.return_codes import ExitCodes


@ClientApplication.subcommand('stop')
class Stop(UPaaSApplication):

    DESCRIPTION = "Stop application"

    def main(self, name):
        self.setup_logger()
        self.log.info("Getting app '%s' details" % name)

        self.api_connect(self.parent.config.server.login,
                         self.parent.config.server.apikey,
                         self.parent.config.server.url)

        try:
            resp = self.api.application.get(name=name)
        except SlumberHttpBaseException as e:
            self.handle_error(e)
        else:
            if not resp.get('objects'):
                self.log.error("No such application registered: %s" % name)
                return ExitCodes.notfound_error

            app = resp['objects'][0]

            try:
                self.api.application(app['id']).stop.put(
                    {'name': app['name']})
            except SlumberHttpBaseException as e:
                self.handle_error(e)
            else:
                self.log.info("Stop task queued")



# -*- coding: utf-8 -*-
"""
    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
    :contact: l.mierzwa@gmail.com
"""


from __future__ import unicode_literals

from plumbum import cli

from slumber.exceptions import SlumberHttpBaseException

from upaas.cli.base import UPaaSApplication

from upaas_client.main import ClientApplication
from upaas_client.return_codes import ExitCodes


@ClientApplication.subcommand('build')
class Build(UPaaSApplication):

    DESCRIPTION = "Build new application package"

    force_fresh = False
    interpreter_version = ''

    @cli.switch(["f", "force-fresh"], help="Force building fresh package")
    def set_force_fresh(self):
        self.force_fresh = True

    @cli.switch(["v", "--interpreter-version"], str,
                help="Force interpreter version (only used for fresh "
                     "packages)")
    def set_interpreter_version(self, value):
        self.interpreter_version = value

    def main(self, name):
        self.setup_logger()
        self.log.info("Getting app '%s' details" % name)

        self.api_connect(self.parent.config.server.login,
                         self.parent.config.server.apikey,
                         self.parent.config.server.url)

        try:
            resp = self.api.application.get(name=name)
        except SlumberHttpBaseException as e:
            self.handle_error(e)
        else:
            if not resp.get('objects'):
                self.log.error("No such application registered: %s" % name)
                return ExitCodes.notfound_error

            app = resp['objects'][0]

            try:
                self.api.application(app['id']).build.put(
                    {'name': app['name']}, force_fresh=int(self.force_fresh),
                    interpreter_version=self.interpreter_version)
            except SlumberHttpBaseException as e:
                self.handle_error(e)
            else:
                self.log.info("Build task queued (fresh: "
                              "%s)" % self.force_fresh)



# -*- coding: utf-8 -*-
"""
    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
    :contact: l.mierzwa@gmail.com
"""


from __future__ import unicode_literals

from upaas.cli.base import UPaaSApplication

from upaas_client.main import ClientApplication
from upaas_client.return_codes import ExitCodes


@ClientApplication.subcommand('list')
class List(UPaaSApplication):

    DESCRIPTION = "List registered applications"

    def main(self):
        self.setup_logger()
        self.log.info("Getting list of registered applications")

        self.api_connect(self.parent.config.server.login,
                         self.parent.config.server.apikey,
                         self.parent.config.server.url)

        try:
            resp = self.api.application.get()
            self.print_msg("%d application(s) "
                           "registered:" % resp['meta']['total_count'])
            for app in resp['objects']:
                self.print_msg(app['name'], prefix='*')
        except Exception as e:
            self.handle_error(e)
            return ExitCodes.command_error



# -*- coding: utf-8 -*-
"""
    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
    :contact: l.mierzwa@gmail.com
"""


from .register import Register
from .list import List
from .show import Show
from .update import Update
from .build import Build
from .start import Start
from .stop import Stop
from .edit import Edit



# -*- coding: utf-8 -*-
"""
    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
    :contact: l.mierzwa@gmail.com
"""


from os.path import expanduser, basename, dirname, exists

from upaas.cli.base import UPaaSApplication
from upaas.config.base import load_config

from plumbum import cli

from upaas_client import __version__ as UPAAS_VERSION
from upaas_client.return_codes import ExitCodes
from upaas_client.config import ClientConfig


class ClientApplication(UPaaSApplication):

    VERSION = UPAAS_VERSION

    config = None
    config_path = None

    @cli.switch(["c", "config"], str, help="Config file to use")
    def set_config_path(self, config_path):
        self.config_path = config_path

    def main(self, *args):
        self.setup_logger()

        if self.config_path:
            if exists(self.config_path):
                config_name = basename(self.config_path)
                config_dir = dirname(self.config_path)
                self.config = load_config(ClientConfig, config_name,
                                          directories=[config_dir])
            else:
                self.log.error("Config not found: %s" % self.config_path)
        else:
            self.config = load_config(ClientConfig, ".upaas.yml",
                                      directories=[".", expanduser("~")])
        if not self.config:
            self.log.error("Missing config file")
            return ExitCodes.missing_config

        if args:
            self.log.error("Unknown command '%s'" % ' '.join(args))
            return ExitCodes.command_error

        if not self.nested_command:
            self.help()
            return ExitCodes.command_error



# -*- coding: utf-8 -*-
"""
    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
    :contact: l.mierzwa@gmail.com
"""


from upaas.config import base


class ClientConfig(base.Config):

    schema = {
        "server": {
            "url": base.StringEntry(required=True),
            "login": base.StringEntry(required=True),
            "apikey": base.StringEntry(required=True),
        }
    }



# -*- coding: utf-8 -*-
"""
    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
    :contact: l.mierzwa@gmail.com
"""


from __future__ import unicode_literals

from slumber.exceptions import SlumberHttpBaseException

from upaas.cli.base import UPaaSApplication
from upaas.config.metadata import MetadataConfig

from upaas_client.main import ClientApplication
from upaas_client.return_codes import ExitCodes


@ClientApplication.subcommand('register')
class Register(UPaaSApplication):

    DESCRIPTION = "Register new application"

    def main(self, name, metadata_path):
        self.setup_logger()
        self.log.info("Registering new application using metadata at "
                      "%s" % metadata_path)

        try:
            with open(metadata_path) as m:
                meta = m.read()
            MetadataConfig.from_file(metadata_path)
        except:
            self.log.error("Invalid metadata in %s" % metadata_path)
            return ExitCodes.command_error

        self.api_connect(self.parent.config.server.login,
                         self.parent.config.server.apikey,
                         self.parent.config.server.url)

        try:
            self.api.application.post({'name': name,
                                       'metadata': meta})
        except SlumberHttpBaseException as e:
            self.handle_error(e)
            return ExitCodes.command_error
        else:
            self.log.info("Application '%s' created successfully" % name)
